package com.cpms.project.training.service;


import com.cpms.project.training.domain.TTrainRoster;

import java.util.List;

/**
 * 培训签字Service接口
 * 
 * @author ruoyi
 * @date 2023-08-03
 */
public interface ITTrainRosterService 
{
    /**
     * 查询培训签字
     * 
     * @param id 培训签字主键
     * @return 培训签字
     */
    public TTrainRoster selectTTrainRosterById(Long id);

    /**
     * 查询培训签字列表
     * 
     * @param tTrainRoster 培训签字
     * @return 培训签字集合
     */
    public List<TTrainRoster> selectTTrainRosterList(TTrainRoster tTrainRoster);

    /**
     * 新增培训签字
     * 
     * @param tTrainRoster 培训签字
     * @return 结果
     */
    public int insertTTrainRoster(TTrainRoster tTrainRoster);

    /**
     * 修改培训签字
     * 
     * @param tTrainRoster 培训签字
     * @return 结果
     */
    public int updateTTrainRoster(TTrainRoster tTrainRoster);

    /**
     * 批量删除培训签字
     * 
     * @param ids 需要删除的培训签字主键集合
     * @return 结果
     */
    public int deleteTTrainRosterByIds(Long[] ids);

    /**
     * 删除培训签字信息
     * 
     * @param id 培训签字主键
     * @return 结果
     */
    public int deleteTTrainRosterById(Long id);
}
